package com.muyu.config.runner;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.benmanes.caffeine.cache.Cache;
import com.muyu.cache.model.MessageTemplateCacheModel;
import com.muyu.config.tenant.CustomTenantHandler;
import com.muyu.vehicle.VehicleInstance;
import com.muyu.vehicle.core.LocalContainer;
import com.muyu.web.common.pool.FixedThreadPool;
import com.muyu.web.common.pool.ScheduledThreadPool;
import com.muyu.web.domain.VehicleInfo;
import com.muyu.web.service.MessageTemplateService;
import com.muyu.web.service.MessageTemplateValueService;
import com.muyu.web.service.VehicleInfoService;
import com.muyu.web.service.VehicleInstanceService;
import jakarta.annotation.PreDestroy;
import jakarta.annotation.Resource;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.annotation.Configuration;

import java.util.List;

/**
 * 车辆
 * @author DongZeLiang
 * @version 1.0
 * @description 车辆
 * @date 2023/11/9
 */
@Log4j2
@Configuration
public class VehicleConfigRunner implements ApplicationRunner {

    @Autowired
    private VehicleInfoService vehicleInfoService;

    @Autowired
    private VehicleInstanceService vehicleInstanceService;

    @Autowired
    private CustomTenantHandler customTenantHandler;

    /**
     * 报文模板缓存
     */
    @Resource(name = "messageTemplateCache")
    private Cache<Long, MessageTemplateCacheModel> messageTemplateCache;

    /**
     * 报文模板业务类
     */
    @Autowired
    private MessageTemplateService messageTemplateService;

    /**
     * 报文模板业务值类
     */
    @Autowired
    private MessageTemplateValueService messageTemplateValueService;


    public void initMessageTemplate() {
        long startTime = System.currentTimeMillis();
        log.info("开始初始化加载报文模板---------------");
        messageTemplateService
                .list()
                .stream()
                .map(messageTemplate -> MessageTemplateCacheModel.messageTemplateBuild(messageTemplate, messageTemplateValueService::valueListByMsgId))
                .filter(model -> {
                    log.info(
                            "过滤校验报文模板[{}-{}] 报文值个数-[{}个] -- {}",
                            model.getKey(), model.getMessageName(), model.getMessageTemplateValueList().size(),
                            model.getMessageTemplateValueList().isEmpty() ? "不合格，已过滤" : "合格"
                    );
                    return model.getMessageTemplateValueList().isEmpty();
                })
                .forEach(model -> messageTemplateCache.put(model.getKey(), model));
        log.info("结束初始化加载报文模板耗时：[{}MS]---------------", System.currentTimeMillis() - startTime);
    }

    /**
     * 初始化加载汽车数据到内存当中
     */
    public void vehiclePageInit (){
        long startTime = System.currentTimeMillis();
        int page = 0, pageSize = 10;
        log.info("初始开始，批量从数据库当中加载数据到内存当中,每次[{}]条", pageSize);
        while (true){
            Page<VehicleInfo> vehiclePage = vehicleInfoService.page(new Page<>(page++, pageSize));
            List<VehicleInfo> vehicleInfoList = vehiclePage.getRecords();
            vehicleInfoList.forEach(vehicleInstanceService::init);
            log.debug("第[{}]页，[{}]条", page, vehicleInfoList.size());
            if (vehicleInfoList.size() < pageSize){
                break;
            }
        }
        log.info("数据加载完成，耗时：{} MS", System.currentTimeMillis() - startTime);
    }

    @Override
    public void run (ApplicationArguments args) {
        customTenantHandler.ignore();
        this.initMessageTemplate();
        this.vehiclePageInit();
        customTenantHandler.remove();
        // 提交给线程池 一分钟 执行一次
//        ThreadPool.submit(new Thread(vehicleService::syncDb), 30);
    }

    /**
     * 关闭程序前操作
     */
    @PreDestroy
    public void destroy(){
        log.info("数据库同步");
        vehicleInfoService.syncDb();

        log.info("下线所有车辆");
        List<VehicleInstance> onlineVehicleInstanceList = LocalContainer.getTenantOnlineVehicleInstance();
        onlineVehicleInstanceList.forEach(VehicleInstance::closeClient);

        log.info("关闭线程池");
        ScheduledThreadPool.shutdown();
        FixedThreadPool.shutDown();

    }
}
